/*
 * main.cpp
 *
 */

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <vector>
#include <stdio.h>
#include <stdint.h>
#include <time.h>
#include <math.h>
#include <sys/types.h>
#include <dirent.h>

#include <SDL.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include <SDL2_rotozoom.h>

#if 0
#include "EmotivAffectivEEG.h"
#endif

#include "FMSoundSynthesis.h"

#if 0
#include <windows.h>

#include "MIDIController.h"
#include "SinusoidGenerator.h"
#include "timing.h"
#include "DataSource.h"
#endif


bool loadWords(std::string filename, std::vector<std::string>& words);
bool loadPictures(std::vector<std::string>& pictures);

bool measureColor(SDL_Surface* image, SDL_Color& averageColor);



int main(int argc, char** argv)
{
  printf("Resonanz 0.001 <nop@iki.fi>\n");
  
  srand(time(0));
  SDL_Init(SDL_INIT_EVERYTHING);

  if(TTF_Init() != 0){
    printf("TTF_Init failed: %s\n", TTF_GetError());
    return -1;
  }
  
  int flags = IMG_INIT_JPG | IMG_INIT_PNG;
  
  if(IMG_Init(flags) != flags){
    printf("IMG_Init failed: %s\n", IMG_GetError());
    IMG_Quit();
    return -1;
  }
  
  TTF_Font* font = 0;
  font = TTF_OpenFont("FreeSans.ttf", 100);
  if(font == 0){
    printf("TTF_OpenFont failure: %s\n", TTF_GetError());
    IMG_Quit();
    return -1;
  }
  
  std::vector<std::string> words;
  
  if(!loadWords("keywords.txt", words)){
    printf("Couldn't load keywords file: %d entries loaded\n", words.size());
    TTF_CloseFont(font);
    IMG_Quit();
    return -1;
  }
  
  printf("%d keywords loaded.\n", words.size());
  
  std::vector<std::string> pictures;
  
  if(!loadPictures(pictures)){
    printf("Couldn't find any pictures: %d entries loaded.\n", pictures.size());
    TTF_CloseFont(font);
    IMG_Quit();
    return -1;
  }
  
  printf("%d pictures loaded.\n", pictures.size());
  
  
  SDL_Window* window = 0;
  const int WIDTH  = 640;
  const int HEIGHT = 480;
  
  window = SDL_CreateWindow("Resonanz", 
			    SDL_WINDOWPOS_CENTERED,
			    SDL_WINDOWPOS_CENTERED,
			    WIDTH, HEIGHT,
			    SDL_WINDOW_SHOWN);

  if(window == 0){
    printf("Could not create window.\n");
    SDL_DestroyWindow(window);
    TTF_CloseFont(font);
    IMG_Quit();
    return 1;
  }
  
  // EmotivAffectivEEG* eeg = new EmotivAffectivEEG();
  SDLSoundSynthesis* fm  = new FMSoundSynthesis();
  // SDLSoundSynthesis* fm = new AdditiveSoundSynthesis();
  
  std::vector<float> parameters;
  parameters.resize(fm->getNumberOfParameters());

  FILE* handle = fopen("session.log", "a+");
  if(handle == 0){
    // delete eeg;
    delete fm;
    SDL_DestroyWindow(window);
    TTF_CloseFont(font);
    IMG_Quit();
    
    printf("ERROR: Cannot open session log file.\n");
    
    return -1;
  }
  

  bool keypress = false;
  
  while(!keypress)
  {
    parameters[0] = 0.5; // (rand() / (double)RAND_MAX);
    parameters[1] = 1000.0*(rand() / (double)RAND_MAX);
    parameters[2] = 1000.0*(rand() / (double)RAND_MAX);
    parameters[3] = 1000.0*(rand() / (double)RAND_MAX);
    
    fm->setParameters(parameters);
    fm->reset();
    // eeg->clearData();
    
    // DISABLE sound for now
    // fm->play();
    
    SDL_Surface* surface = SDL_GetWindowSurface(window);
    // SDL_LockSurface(surface);    
    
    SDL_FillRect(surface, NULL, SDL_MapRGB(surface->format, 0, 0, 0));
    
    const unsigned int index1 = rand() % pictures.size();
    
    SDL_Surface* image = IMG_Load(pictures[index1].c_str());

    SDL_Color averageColor;
    measureColor(image, averageColor);
    int bgcolor = 
      (int)(averageColor.r + averageColor.g + averageColor.b)/3;
    
    if(image != 0){
      SDL_Rect imageRect;      
      SDL_Surface* scaled = 0;
      
      if((image->w) > (image->h)){
	double wscale = ((double)WIDTH)/((double)image->w);
	scaled = zoomSurface(image, wscale, wscale, SMOOTHING_ON);
      }
      else{
	double hscale = ((double)HEIGHT)/((double)image->h);
	scaled = zoomSurface(image, hscale, hscale, SMOOTHING_ON);
      }
      
      imageRect.w = scaled->w;
      imageRect.h = scaled->h;
      imageRect.x = (WIDTH - scaled->w)/2;
      imageRect.y = (HEIGHT - scaled->h)/2;
      
      SDL_BlitSurface(scaled, NULL, surface, &imageRect);

      SDL_FreeSurface(image);
      SDL_FreeSurface(scaled);
    }

    SDL_Color white = { 255, 255, 255 };
    SDL_Color red   = { 255, 0  , 0   };
    SDL_Color green = { 0  , 255, 0   };
    SDL_Color blue  = { 0  , 0  , 255 };
    SDL_Color black = { 0  , 0  , 0   };
    SDL_Color color;
    
    color = white;
    if(bgcolor > 200)
      color = black;

    const unsigned int index2 = rand() % words.size();
    
    SDL_Surface* msg = 
      TTF_RenderText_Blended(font, words[index2].c_str(), color);
    
    SDL_Rect messageRect;
    messageRect.x = (WIDTH - msg->w)/2;
    messageRect.y = (HEIGHT - msg->h)/2;
    messageRect.w = msg->w;
    messageRect.h = msg->h;

    SDL_BlitSurface(msg, NULL, surface, &messageRect);
    
    SDL_FreeSurface(msg);
    
    // SDL_UnlockSurface(surface);
    SDL_UpdateWindowSurface(window);
    
    // microsleep(20000); // listen sound for 20 seconds
    // SDL_Delay(500); // 5 secs is good time for measure sound response patterns
    SDL_Delay(250);
    
    fm->pause();
    
    
    /////////////////////////////////////////////////////////
    // check for keypress (for window, not console)
    SDL_Event event;
    while(SDL_PollEvent(&event)){
      if(event.type == SDL_KEYDOWN || event.type == SDL_KEYUP){
	keypress = true;
      }
    }
    
  }
  
  fm->pause();
  
  delete fm;
  SDL_DestroyWindow(window);
  // delete eeg;
  fclose(handle);

  TTF_CloseFont(font);
  IMG_Quit();
  TTF_Quit();
  SDL_Quit();
  
  return 0;
}


bool loadWords(std::string filename, std::vector<std::string>& words)
{
  FILE* handle = fopen(filename.c_str(), "rt");
  
  if(handle == 0)
    return false;
  
  char buffer[256];
  
  while(fgets(buffer, 256, handle) == buffer){
    const int N = strlen(buffer);
    
    for(int i=0;i<N;i++)
      if(buffer[i] == '\n')
	buffer[i] = '\0';
    
    words.push_back(buffer);
  }
  
  fclose(handle);
  
  return true;
}


bool loadPictures(std::vector<std::string>& pictures)
{
  // looks for pics/*.jpg and pics/*.png files
  
  DIR* dp;
  struct dirent *ep;
  
  dp = opendir("pics/");
  
  if(dp == 0) return false;
  
  while((ep = readdir(dp)) != NULL){
    if(ep->d_name == NULL) continue;    
    std::string name = ep->d_name;
    
    if(name.find(".jpg") == (name.size() - 4) || 
       name.find(".png") == (name.size() - 4))
    {
      std::string fullname = "pics/";
      fullname = fullname + name;
      
      pictures.push_back(fullname);
    }
  }
  
  closedir(dp);
  
  return true;
}


bool measureColor(SDL_Surface* image, SDL_Color& averageColor)
{
  double r = 0.0;
  double g = 0.0;
  double b = 0.0;
  unsigned int N = 0;
  
  SDL_Surface* im =
    SDL_ConvertSurfaceFormat(image, SDL_PIXELFORMAT_ARGB8888, 0);
  
  if(im == 0) return false;
  
  unsigned int* buffer = (unsigned int*)im->pixels;
  
  for(int y=0;y<(im->h);y++){
    for(int x=0;x<(im->w);x++){
      int rr = (buffer[x + y*(image->pitch/4)] & 0xFF0000) >> 16;
      int gg = (buffer[x + y*(image->pitch/4)] & 0x00FF00) >> 8;
      int bb = (buffer[x + y*(image->pitch/4)] & 0x0000FF) >> 0;
      
      r += rr;
      g += gg;
      b += bb;
      
      N++;
    }
  }
  
  r /= N;
  g /= N;
  b /= N;
  
  averageColor.r = (Uint8)r;
  averageColor.g = (Uint8)g;
  averageColor.b = (Uint8)b;

  SDL_FreeSurface(im);
  
  return true;
}

